@@ -17,13 +17,11 @@
 # under the License.
 from __future__ import annotations
 
-import fcntl
 import importlib
 import inspect
 import json
 import logging
 import os
-import pickle
 import shutil
 import subprocess
 import sys
@@ -36,7 +34,7 @@ from pathlib import Path
 from tempfile import TemporaryDirectory
 from typing import TYPE_CHECKING, Any, Callable, Collection, Iterable, Mapping, NamedTuple, Sequence, cast
 
-import dill
+import lazy_object_proxy
 
 from airflow.compat.functools import cache
 from airflow.exceptions import (
@@ -51,13 +49,16 @@ from airflow.models.skipmixin import SkipMixin
 from airflow.models.taskinstance import _CURRENT_CONTEXT
 from airflow.models.variable import Variable
 from airflow.operators.branch import BranchMixIn
+from airflow.typing_compat import Literal
 from airflow.utils import hashlib_wrapper
-from airflow.utils.context import context_copy_partial, context_merge
+from airflow.utils.context import context_copy_partial, context_get_outlet_events, context_merge
 from airflow.utils.file import get_unique_dag_module_name
-from airflow.utils.operator_helpers import KeywordParameters
+from airflow.utils.operator_helpers import ExecutionCallableRunner, KeywordParameters
 from airflow.utils.process_utils import execute_in_subprocess
 from airflow.utils.python_virtualenv import prepare_virtualenv, write_python_script
 
+log = logging.getLogger(__name__)
+
 if TYPE_CHECKING:
     from pendulum.datetime import DateTime
 
@@ -76,7 +77,8 @@ def is_venv_installed() -> bool:
 
 
 def task(python_callable: Callable | None = None, multiple_outputs: bool | None = None, **kwargs):
-    """Use :func:`airflow.decorators.task` instead, this is deprecated.
+    """
+    Use :func:`airflow.decorators.task` instead, this is deprecated.
 
     Calls ``@task.python`` and allows users to turn a Python function into
     an Airflow task.
@@ -231,6 +233,7 @@ class PythonOperator(BaseOperator):
     def execute(self, context: Context) -> Any:
         context_merge(context, self.op_kwargs, templates_dict=self.templates_dict)
         self.op_kwargs = self.determine_kwargs(context)
+        self._dataset_events = context_get_outlet_events(context)
 
         return_value = self.execute_callable()
         if self.show_return_value_in_logs:
@@ -249,7 +252,8 @@ class PythonOperator(BaseOperator):
 
         :return: the return value of the call.
         """
-        return self.python_callable(*self.op_args, **self.op_kwargs)
+        runner = ExecutionCallableRunner(self.python_callable, self._dataset_events, logger=self.log)
+        return runner.run(*self.op_args, **self.op_kwargs)
 
 
 class BranchPythonOperator(PythonOperator, BranchMixIn):
@@ -257,12 +261,13 @@ class BranchPythonOperator(PythonOperator, BranchMixIn):
     A workflow can "branch" or follow a path after the execution of this task.
 
     It derives the PythonOperator and expects a Python function that returns
-    a single task_id or list of task_ids to follow. The task_id(s) returned
-    should point to a task directly downstream from {self}. All other "branches"
-    or directly downstream tasks are marked with a state of ``skipped`` so that
-    these paths can't move forward. The ``skipped`` states are propagated
-    downstream to allow for the DAG state to fill up and the DAG run's state
-    to be inferred.
+    a single task_id, a single task_group_id, or a list of task_ids and/or
+    task_group_ids to follow. The task_id(s) and/or task_group_id(s) returned
+    should point to a task or task group directly downstream from {self}. All
+    other "branches" or directly downstream tasks are marked with a state of
+    ``skipped`` so that these paths can't move forward. The ``skipped`` states
+    are propagated downstream to allow for the DAG state to fill up and
+    the DAG run's state to be inferred.
     """
 
     def execute(self, context: Context) -> Any:
@@ -341,6 +346,41 @@ class ShortCircuitOperator(PythonOperator, SkipMixin):
         return condition
 
 
+def _load_pickle():
+    import pickle
+
+    return pickle
+
+
+def _load_dill():
+    try:
+        import dill
+    except ModuleNotFoundError:
+        log.error("Unable to import `dill` module. Please please make sure that it installed.")
+        raise
+    return dill
+
+
+def _load_cloudpickle():
+    try:
+        import cloudpickle
+    except ModuleNotFoundError:
+        log.error(
+            "Unable to import `cloudpickle` module. "
+            "Please install it with: pip install 'apache-airflow[cloudpickle]'"
+        )
+        raise
+    return cloudpickle
+
+
+_SerializerTypeDef = Literal["pickle", "cloudpickle", "dill"]
+_SERIALIZERS: dict[_SerializerTypeDef, Any] = {
+    "pickle": lazy_object_proxy.Proxy(_load_pickle),
+    "dill": lazy_object_proxy.Proxy(_load_dill),
+    "cloudpickle": lazy_object_proxy.Proxy(_load_cloudpickle),
+}
+
+
 class _BasePythonVirtualenvOperator(PythonOperator, metaclass=ABCMeta):
     BASE_SERIALIZABLE_CONTEXT_KEYS = {
         "ds",
@@ -391,7 +431,7 @@ class _BasePythonVirtualenvOperator(PythonOperator, metaclass=ABCMeta):
         self,
         *,
         python_callable: Callable,
-        use_dill: bool = False,
+        serializer: _SerializerTypeDef | None = None,
         op_args: Collection[Any] | None = None,
         op_kwargs: Mapping[str, Any] | None = None,
         string_args: Iterable[str] | None = None,
@@ -399,6 +439,9 @@ class _BasePythonVirtualenvOperator(PythonOperator, metaclass=ABCMeta):
         templates_exts: list[str] | None = None,
         expect_airflow: bool = True,
         skip_on_exit_code: int | Container[int] | None = None,
+        env_vars: dict[str, str] | None = None,
+        inherit_env: bool = True,
+        use_dill: bool = False,
         **kwargs,
     ):
         if (
@@ -406,7 +449,9 @@ class _BasePythonVirtualenvOperator(PythonOperator, metaclass=ABCMeta):
             or isinstance(python_callable, types.LambdaType)
             and python_callable.__name__ == "<lambda>"
         ):
-            raise AirflowException("PythonVirtualenvOperator only supports functions for python_callable arg")
+            raise ValueError(f"{type(self).__name__} only supports functions for python_callable arg")
+        if inspect.isgeneratorfunction(python_callable):
+            raise ValueError(f"{type(self).__name__} does not support using 'yield' in python_callable")
         super().__init__(
             python_callable=python_callable,
             op_args=op_args,
@@ -416,8 +461,29 @@ class _BasePythonVirtualenvOperator(PythonOperator, metaclass=ABCMeta):
             **kwargs,
         )
         self.string_args = string_args or []
-        self.use_dill = use_dill
-        self.pickling_library = dill if self.use_dill else pickle
+
+        if use_dill:
+            warnings.warn(
+                "`use_dill` is deprecated and will be removed in a future version. "
+                "Please provide serializer='dill' instead.",
+                RemovedInAirflow3Warning,
+                stacklevel=3,
+            )
+            if serializer:
+                raise AirflowException(
+                    "Both 'use_dill' and 'serializer' parameters are set. Please set only one of them"
+                )
+            serializer = "dill"
+        serializer = serializer or "pickle"
+        if serializer not in _SERIALIZERS:
+            msg = (
+                f"Unsupported serializer {serializer!r}. "
+                f"Expected one of {', '.join(map(repr, _SERIALIZERS))}"
+            )
+            raise AirflowException(msg)
+        self.pickling_library = _SERIALIZERS[serializer]
+        self.serializer: _SerializerTypeDef = serializer
+
         self.expect_airflow = expect_airflow
         self.skip_on_exit_code = (
             skip_on_exit_code
@@ -426,6 +492,8 @@ class _BasePythonVirtualenvOperator(PythonOperator, metaclass=ABCMeta):
             if skip_on_exit_code is not None
             else []
         )
+        self.env_vars = env_vars
+        self.inherit_env = inherit_env
 
     @abstractmethod
     def _iter_serializable_context_keys(self):
@@ -442,6 +510,7 @@ class _BasePythonVirtualenvOperator(PythonOperator, metaclass=ABCMeta):
 
     def _write_args(self, file: Path):
         if self.op_args or self.op_kwargs:
+            self.log.info("Use %r as serializer.", self.serializer)
             file.write_bytes(self.pickling_library.dumps({"args": self.op_args, "kwargs": self.op_kwargs}))
 
     def _write_string_args(self, file: Path):
@@ -479,7 +548,7 @@ class _BasePythonVirtualenvOperator(PythonOperator, metaclass=ABCMeta):
                 "op_args": self.op_args,
                 "op_kwargs": op_kwargs,
                 "expect_airflow": self.expect_airflow,
-                "pickling_library": self.pickling_library.__name__,
+                "pickling_library": self.serializer,
                 "python_callable": self.python_callable.__name__,
                 "python_callable_source": self.get_python_source(),
             }
@@ -493,6 +562,10 @@ class _BasePythonVirtualenvOperator(PythonOperator, metaclass=ABCMeta):
                 render_template_as_native_obj=self.dag.render_template_as_native_obj,
             )
 
+            env_vars = dict(os.environ) if self.inherit_env else {}
+            if self.env_vars:
+                env_vars.update(self.env_vars)
+
             try:
                 execute_in_subprocess(
                     cmd=[
@@ -502,7 +575,8 @@ class _BasePythonVirtualenvOperator(PythonOperator, metaclass=ABCMeta):
                         os.fspath(output_path),
                         os.fspath(string_args_path),
                         os.fspath(termination_log_path),
-                    ]
+                    ],
+                    env=env_vars,
                 )
             except subprocess.CalledProcessError as e:
                 if e.returncode in self.skip_on_exit_code:
@@ -548,9 +622,13 @@ class PythonVirtualenvOperator(_BasePythonVirtualenvOperator):
         "requirements file" as specified by pip.
     :param python_version: The Python version to run the virtual environment with. Note that
         both 2 and 2.7 are acceptable forms.
-    :param use_dill: Whether to use dill to serialize
-        the args and result (pickle is default). This allow more complex types
-        but requires you to include dill in your requirements.
+    :param serializer: Which serializer use to serialize the args and result. It can be one of the following:
+
+        - ``"pickle"``: (default) Use pickle for serialization. Included in the Python Standard Library.
+        - ``"cloudpickle"``: Use cloudpickle for serialize more complex types,
+          this requires to include cloudpickle in your requirements.
+        - ``"dill"``: Use dill for serialize more complex types,
+          this requires to include dill in your requirements.
     :param system_site_packages: Whether to include
         system_site_packages in your virtual environment.
         See virtualenv documentation for more information.
@@ -579,6 +657,15 @@ class PythonVirtualenvOperator(_BasePythonVirtualenvOperator):
         virtual environment will be cached, creates a sub-folder venv-{hash} whereas hash will be replaced
         with a checksum of requirements. If not provided the virtual environment will be created and deleted
         in a temp folder for every execution.
+    :param env_vars: A dictionary containing additional environment variables to set for the virtual
+        environment when it is executed.
+    :param inherit_env: Whether to inherit the current environment variables when executing the virtual
+        environment. If set to ``True``, the virtual environment will inherit the environment variables
+        of the parent process (``os.environ``). If set to ``False``, the virtual environment will be
+        executed with a clean environment.
+    :param use_dill: Deprecated, use ``serializer`` instead. Whether to use dill to serialize
+        the args and result (pickle is default). This allows more complex types
+        but requires you to include dill in your requirements.
     """
 
     template_fields: Sequence[str] = tuple(
@@ -592,7 +679,7 @@ class PythonVirtualenvOperator(_BasePythonVirtualenvOperator):
         python_callable: Callable,
         requirements: None | Iterable[str] | str = None,
         python_version: str | None = None,
-        use_dill: bool = False,
+        serializer: _SerializerTypeDef | None = None,
         system_site_packages: bool = True,
         pip_install_options: list[str] | None = None,
         op_args: Collection[Any] | None = None,
@@ -604,6 +691,9 @@ class PythonVirtualenvOperator(_BasePythonVirtualenvOperator):
         skip_on_exit_code: int | Container[int] | None = None,
         index_urls: None | Collection[str] | str = None,
         venv_cache_path: None | os.PathLike[str] = None,
+        env_vars: dict[str, str] | None = None,
+        inherit_env: bool = True,
+        use_dill: bool = False,
         **kwargs,
     ):
         if (
@@ -643,7 +733,7 @@ class PythonVirtualenvOperator(_BasePythonVirtualenvOperator):
         self.venv_cache_path = venv_cache_path
         super().__init__(
             python_callable=python_callable,
-            use_dill=use_dill,
+            serializer=serializer,
             op_args=op_args,
             op_kwargs=op_kwargs,
             string_args=string_args,
@@ -651,14 +741,24 @@ class PythonVirtualenvOperator(_BasePythonVirtualenvOperator):
             templates_exts=templates_exts,
             expect_airflow=expect_airflow,
             skip_on_exit_code=skip_on_exit_code,
+            env_vars=env_vars,
+            inherit_env=inherit_env,
+            use_dill=use_dill,
             **kwargs,
         )
 
-    def _requirements_list(self) -> list[str]:
+    def _requirements_list(self, exclude_cloudpickle: bool = False) -> list[str]:
         """Prepare a list of requirements that need to be installed for the virtual environment."""
         requirements = [str(dependency) for dependency in self.requirements]
-        if not self.system_site_packages and self.use_dill and "dill" not in requirements:
-            requirements.append("dill")
+        if not self.system_site_packages:
+            if (
+                self.serializer == "cloudpickle"
+                and not exclude_cloudpickle
+                and "cloudpickle" not in requirements
+            ):
+                requirements.append("cloudpickle")
+            elif self.serializer == "dill" and "dill" not in requirements:
+                requirements.append("dill")
         requirements.sort()  # Ensure a hash is stable
         return requirements
 
@@ -675,8 +775,9 @@ class PythonVirtualenvOperator(_BasePythonVirtualenvOperator):
             index_urls=self.index_urls,
         )
 
-    def _calculate_cache_hash(self) -> tuple[str, str]:
-        """Generate the hash of the cache folder to use.
+    def _calculate_cache_hash(self, exclude_cloudpickle: bool = False) -> tuple[str, str]:
+        """
+        Generate the hash of the cache folder to use.
 
         The following factors are used as input for the hash:
         - (sorted) list of requirements
@@ -689,7 +790,7 @@ class PythonVirtualenvOperator(_BasePythonVirtualenvOperator):
         Returns a hash and the data dict which is the base for the hash as text.
         """
         hash_dict = {
-            "requirements_list": self._requirements_list(),
+            "requirements_list": self._requirements_list(exclude_cloudpickle=exclude_cloudpickle),
             "pip_install_options": self.pip_install_options,
             "index_urls": self.index_urls,
             "cache_key": str(Variable.get("PythonVirtualenvOperator.cache_key", "")),
@@ -709,6 +810,8 @@ class PythonVirtualenvOperator(_BasePythonVirtualenvOperator):
         venv_path.parent.mkdir(parents=True, exist_ok=True)
         with open(f"{venv_path}.lock", "w") as f:
             # Ensure that cache is not build by parallel workers
+            import fcntl
+
             fcntl.flock(f, fcntl.LOCK_EX)
 
             hash_marker = venv_path / "install_complete_marker.json"
@@ -717,17 +820,25 @@ class PythonVirtualenvOperator(_BasePythonVirtualenvOperator):
                     if hash_marker.exists():
                         previous_hash_data = hash_marker.read_text(encoding="utf8")
                         if previous_hash_data == hash_data:
-                            self.log.info("Re-using cached Python virtual environment in %s", venv_path)
+                            self.log.info("Reusing cached Python virtual environment in %s", venv_path)
                             return venv_path
 
-                        self.log.error(
-                            "Unicorn alert: Found a previous virtual environment in %s "
-                            "with the same hash but different parameters. Previous setup: '%s' / "
-                            "Requested venv setup: '%s'. Please report a bug to airflow!",
-                            venv_path,
-                            previous_hash_data,
-                            hash_data,
-                        )
+                        _, hash_data_before_upgrade = self._calculate_cache_hash(exclude_cloudpickle=True)
+                        if previous_hash_data == hash_data_before_upgrade:
+                            self.log.warning(
+                                "Found a previous virtual environment in  with outdated dependencies %s, "
+                                "deleting and re-creating.",
+                                venv_path,
+                            )
+                        else:
+                            self.log.error(
+                                "Unicorn alert: Found a previous virtual environment in %s "
+                                "with the same hash but different parameters. Previous setup: '%s' / "
+                                "Requested venv setup: '%s'. Please report a bug to airflow!",
+                                venv_path,
+                                previous_hash_data,
+                                hash_data,
+                            )
                     else:
                         self.log.warning(
                             "Found a previous (probably partial installed) virtual environment in %s, "
@@ -773,12 +884,13 @@ class BranchPythonVirtualenvOperator(PythonVirtualenvOperator, BranchMixIn):
     A workflow can "branch" or follow a path after the execution of this task in a virtual environment.
 
     It derives the PythonVirtualenvOperator and expects a Python function that returns
-    a single task_id or list of task_ids to follow. The task_id(s) returned
-    should point to a task directly downstream from {self}. All other "branches"
-    or directly downstream tasks are marked with a state of ``skipped`` so that
-    these paths can't move forward. The ``skipped`` states are propagated
-    downstream to allow for the DAG state to fill up and the DAG run's state
-    to be inferred.
+    a single task_id, a single task_group_id, or a list of task_ids and/or
+    task_group_ids to follow. The task_id(s) and/or task_group_id(s) returned
+    should point to a task or task group directly downstream from {self}. All
+    other "branches" or directly downstream tasks are marked with a state of
+    ``skipped`` so that these paths can't move forward. The ``skipped`` states
+    are propagated downstream to allow for the DAG state to fill up and
+    the DAG run's state to be inferred.
 
     .. seealso::
         For more information on how to use this operator, take a look at the guide:
@@ -816,10 +928,14 @@ class ExternalPythonOperator(_BasePythonVirtualenvOperator):
         a virtual environment that should be used (in ``VENV/bin`` folder). Should be absolute path
         (so usually start with "/" or "X:/" depending on the filesystem/os used).
     :param python_callable: A python function with no references to outside variables,
-        defined with def, which will be run in a virtual environment
-    :param use_dill: Whether to use dill to serialize
-        the args and result (pickle is default). This allow more complex types
-        but if dill is not preinstalled in your virtual environment, the task will fail with use_dill enabled.
+        defined with def, which will be run in a virtual environment.
+    :param serializer: Which serializer use to serialize the args and result. It can be one of the following:
+
+        - ``"pickle"``: (default) Use pickle for serialization. Included in the Python Standard Library.
+        - ``"cloudpickle"``: Use cloudpickle for serialize more complex types,
+          this requires to include cloudpickle in your requirements.
+        - ``"dill"``: Use dill for serialize more complex types,
+          this requires to include dill in your requirements.
     :param op_args: A list of positional arguments to pass to python_callable.
     :param op_kwargs: A dict of keyword arguments to pass to python_callable.
     :param string_args: Strings that are present in the global var virtualenv_string_args,
@@ -837,6 +953,15 @@ class ExternalPythonOperator(_BasePythonVirtualenvOperator):
     :param skip_on_exit_code: If python_callable exits with this exit code, leave the task
         in ``skipped`` state (default: None). If set to ``None``, any non-zero
         exit code will be treated as a failure.
+    :param env_vars: A dictionary containing additional environment variables to set for the virtual
+        environment when it is executed.
+    :param inherit_env: Whether to inherit the current environment variables when executing the virtual
+        environment. If set to ``True``, the virtual environment will inherit the environment variables
+        of the parent process (``os.environ``). If set to ``False``, the virtual environment will be
+        executed with a clean environment.
+    :param use_dill: Deprecated, use ``serializer`` instead. Whether to use dill to serialize
+        the args and result (pickle is default). This allows more complex types
+        but requires you to include dill in your requirements.
     """
 
     template_fields: Sequence[str] = tuple({"python"}.union(PythonOperator.template_fields))
@@ -846,7 +971,7 @@ class ExternalPythonOperator(_BasePythonVirtualenvOperator):
         *,
         python: str,
         python_callable: Callable,
-        use_dill: bool = False,
+        serializer: _SerializerTypeDef | None = None,
         op_args: Collection[Any] | None = None,
         op_kwargs: Mapping[str, Any] | None = None,
         string_args: Iterable[str] | None = None,
@@ -855,6 +980,9 @@ class ExternalPythonOperator(_BasePythonVirtualenvOperator):
         expect_airflow: bool = True,
         expect_pendulum: bool = False,
         skip_on_exit_code: int | Container[int] | None = None,
+        env_vars: dict[str, str] | None = None,
+        inherit_env: bool = True,
+        use_dill: bool = False,
         **kwargs,
     ):
         if not python:
@@ -863,7 +991,7 @@ class ExternalPythonOperator(_BasePythonVirtualenvOperator):
         self.expect_pendulum = expect_pendulum
         super().__init__(
             python_callable=python_callable,
-            use_dill=use_dill,
+            serializer=serializer,
             op_args=op_args,
             op_kwargs=op_kwargs,
             string_args=string_args,
@@ -871,6 +999,9 @@ class ExternalPythonOperator(_BasePythonVirtualenvOperator):
             templates_exts=templates_exts,
             expect_airflow=expect_airflow,
             skip_on_exit_code=skip_on_exit_code,
+            env_vars=env_vars,
+            inherit_env=inherit_env,
+            use_dill=use_dill,
             **kwargs,
         )
 
