@@ -764,7 +764,7 @@ def _get_flask_db(sql_database_uri):
     flask_app.config["SQLALCHEMY_DATABASE_URI"] = sql_database_uri
     flask_app.config["SQLALCHEMY_TRACK_MODIFICATIONS"] = False
     db = SQLAlchemy(flask_app)
-    AirflowDatabaseSessionInterface(app=flask_app, db=db, table="session", key_prefix="")
+    AirflowDatabaseSessionInterface(app=flask_app, client=db, table="session", key_prefix="")
     return db
 
 
@@ -1709,6 +1709,7 @@ def resetdb(session: Session = NEW_SESSION, skip_init: bool = False, use_migrati
     with create_global_lock(session=session, lock=DBLocks.MIGRATIONS), connection.begin():
         drop_airflow_models(connection)
         drop_airflow_moved_tables(connection)
+        drop_flask_session_table(connection)
 
     if not skip_init:
         initdb(session=session, use_migration_files=use_migration_files)
@@ -1801,6 +1802,16 @@ def drop_airflow_moved_tables(connection):
         Base.metadata.remove(tbl)
 
 
+def drop_flask_session_table(connection):
+    from airflow.models.base import Base
+
+    tables = set(inspect(connection).get_table_names())
+    to_delete = [Table(x, Base.metadata) for x in tables if x == "session"]
+    for tbl in to_delete:
+        tbl.drop(settings.engine, checkfirst=False)
+        Base.metadata.remove(tbl)
+
+
 @provide_session
 def check(session: Session = NEW_SESSION):
     """
@@ -1951,7 +1962,8 @@ def check_query_exists(query_stmt: Select, *, session: Session) -> bool:
     :meta private:
     """
     count_stmt = select(literal(True)).select_from(query_stmt.order_by(None).subquery())
-    return session.scalar(count_stmt)
+    # we must cast to bool because scalar() can return None
+    return bool(session.scalar(count_stmt))
 
 
 def exists_query(*where: ClauseElement, session: Session) -> bool:
